/*
 * ToGrammarRightRG.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: 9. 2. 2014
 *      Author: Tomas Pecka
 */

#ifndef __TO_GRAMMAR_RIGHT_RG_H__
#define __TO_GRAMMAR_RIGHT_RG_H__

#include <grammar/Regular/RightRG.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

namespace automaton {

namespace convert {

/**
 * Converts a finite automaton to a right regular grammar (Melichar: Jazyky a překlady 2.104).
 */
class ToGrammarRightRG {
public:
	/**
	 * Performs the conversion of the finite automaton to right regular grammar.
	 * @param automaton a finite automaton to convert
	 * @return right regular grammar equivalent to the source @p automaton.
	 */
	static grammar::RightRG < > convert(const automaton::NFA < > & automaton);

	/**
	 * \overload
	 */
	static grammar::RightRG < > convert(const automaton::DFA < > & automaton);
};

} /* namespace convert */

} /* namespace automaton */

#endif /* __TO_GRAMMAR_RIGHT_RG_H__ */
