#include "GrammarToCNFTest.h"

#include "grammar/simplify/ToCNF.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"

#include <factory/StringDataFactory.hpp>

#include <common/DefaultSymbolsPairType.h>

#include <grammar/Grammar.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarToCNFTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarToCNFTest );

void GrammarToCNFTest::setUp() {
}

void GrammarToCNFTest::tearDown() {
}

void GrammarToCNFTest::testToCNFRules1() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType A = DefaultSymbolType("A");
	DefaultSymbolType B = DefaultSymbolType("B");
	DefaultSymbolType C = DefaultSymbolType("C");
	DefaultSymbolType D = DefaultSymbolType("D");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");

	DefaultSymbolType aP = DefaultSymbolType("a'");
	DefaultSymbolType bP = DefaultSymbolType("b'");

	grammar::CFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});

	grammar::CNF < > grammar2 = grammar::simplify::ToCNF::convert(grammar1);

	grammar::CNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, C, D, aP, bP});
	grammar3.setTerminalAlphabet({a, b});
	grammar3.addRule(aP, a);
	grammar3.addRule(bP, b);

	std::cout << grammar2 << std::endl;
	std::cout << grammar3 << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToCNFTest::testToCNFRules2() {
	DefaultSymbolType S = DefaultSymbolType("S");
	DefaultSymbolType X = DefaultSymbolType("X");
	DefaultSymbolType Y = DefaultSymbolType("Y");

	DefaultSymbolType a = DefaultSymbolType("a");
	DefaultSymbolType b = DefaultSymbolType("b");
	DefaultSymbolType c = DefaultSymbolType("c");

	grammar::CFG < > grammar1(S);
	grammar1.setNonterminalAlphabet({S, X, Y});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule({S}, {a, X, b, X});
	grammar1.addRule({X}, {a, Y});
	grammar1.addRule({X}, {b, Y});
	grammar1.addRule({X}, {});
	grammar1.addRule({Y}, {X});
	grammar1.addRule({Y}, {c});

	grammar::CNF < > grammar2 = grammar::simplify::ToCNF::convert(grammar1);
	DefaultSymbolType aP = DefaultSymbolType("a'");
	DefaultSymbolType bP = DefaultSymbolType("b'");
	DefaultSymbolType cP = DefaultSymbolType("c'");
	DefaultSymbolType Xb = DefaultSymbolType(DefaultSymbolType(DefaultSymbolsPairType(ext::make_pair(X, b))));
	DefaultSymbolType aX = DefaultSymbolType(DefaultSymbolType(DefaultSymbolsPairType(ext::make_pair(a, X))));
	DefaultSymbolType bX = DefaultSymbolType(DefaultSymbolType(DefaultSymbolsPairType(ext::make_pair(b, X))));

	grammar::CNF < > grammar3(S);
	grammar3.setNonterminalAlphabet({S, X, Y, aP, bP, cP, Xb, aX, bX});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, ext::make_pair(aX, bX));
	grammar3.addRule(S, ext::make_pair(aP, bX));
	grammar3.addRule(S, ext::make_pair(aP, Xb));
	grammar3.addRule(S, ext::make_pair(aP, bP));
	grammar3.addRule(aX, ext::make_pair(aP, X));
	grammar3.addRule(bX, ext::make_pair(bP, X));
	grammar3.addRule(Xb, ext::make_pair(X, bP));
	grammar3.addRule(X, ext::make_pair(aP, Y));
	grammar3.addRule(X, ext::make_pair(bP, Y));
	grammar3.addRule(X, a);
	grammar3.addRule(X, b);
	grammar3.addRule(Y, ext::make_pair(aP, Y));
	grammar3.addRule(Y, ext::make_pair(bP, Y));
	grammar3.addRule(Y, a);
	grammar3.addRule(Y, b);
	grammar3.addRule(Y, c);
	grammar3.addRule(aP, a);
	grammar3.addRule(bP, b);
	grammar3.addRule(cP, c);

	std::cout << factory::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << factory::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

