#include <alib/list>
#include "rg2rgTest.h"

#include "grammar/convert/ToGrammarLeftRG.h"
#include "grammar/convert/ToGrammarRightRG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( rg2rgTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( rg2rgTest );

void rg2rgTest::setUp() {
}

void rg2rgTest::tearDown() {
}

void rg2rgTest::testConversion() {
  grammar::RightRG < > rrGrammar(DefaultSymbolType(1));

  rrGrammar.addNonterminalSymbol(DefaultSymbolType(1));
  rrGrammar.addNonterminalSymbol(DefaultSymbolType(2));
  rrGrammar.addNonterminalSymbol(DefaultSymbolType(3));
  rrGrammar.addTerminalSymbol(DefaultSymbolType("a"));
  rrGrammar.addTerminalSymbol(DefaultSymbolType("b"));
  
  rrGrammar.addRule(DefaultSymbolType(1), ext::make_pair(DefaultSymbolType("a"), DefaultSymbolType(2)));
  rrGrammar.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType("b"), DefaultSymbolType(3)));
  rrGrammar.addRule(DefaultSymbolType(3), DefaultSymbolType("a"));

  grammar::LeftRG < > lrGrammar = grammar::convert::ToGrammarLeftRG::convert(rrGrammar);

  grammar::LeftRG < > lrGrammarRef(DefaultSymbolType(4));

  lrGrammarRef.addNonterminalSymbol(DefaultSymbolType(1));
  lrGrammarRef.addNonterminalSymbol(DefaultSymbolType(2));
  lrGrammarRef.addNonterminalSymbol(DefaultSymbolType(3));
  lrGrammarRef.addTerminalSymbol(DefaultSymbolType("a"));
  lrGrammarRef.addTerminalSymbol(DefaultSymbolType("b"));

  lrGrammarRef.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType(1), DefaultSymbolType("a")));
  lrGrammarRef.addRule(DefaultSymbolType(2), DefaultSymbolType("a"));
  lrGrammarRef.addRule(DefaultSymbolType(3), ext::make_pair(DefaultSymbolType(2), DefaultSymbolType("b")));
  lrGrammarRef.addRule(DefaultSymbolType(4), ext::make_pair(DefaultSymbolType(3), DefaultSymbolType("a")));

  CPPUNIT_ASSERT(lrGrammarRef == lrGrammar);
}

void rg2rgTest::testConversion2() {
  grammar::LeftRG < > lrGrammar(DefaultSymbolType(4));

  lrGrammar.addNonterminalSymbol(DefaultSymbolType(2));
  lrGrammar.addNonterminalSymbol(DefaultSymbolType(3));
  lrGrammar.addNonterminalSymbol(DefaultSymbolType(4));
  lrGrammar.addTerminalSymbol(DefaultSymbolType("a"));
  lrGrammar.addTerminalSymbol(DefaultSymbolType("b"));

  lrGrammar.addRule(DefaultSymbolType(2), DefaultSymbolType("a"));
  lrGrammar.addRule(DefaultSymbolType(3), ext::make_pair(DefaultSymbolType(2), DefaultSymbolType("b")));
  lrGrammar.addRule(DefaultSymbolType(4), ext::make_pair(DefaultSymbolType(3), DefaultSymbolType("a")));

  grammar::RightRG < > rrGrammar = grammar::convert::ToGrammarRightRG::convert(lrGrammar);

  grammar::RightRG < > rrGrammarRef(DefaultSymbolType(5));

  rrGrammarRef.addNonterminalSymbol(DefaultSymbolType(2));
  rrGrammarRef.addNonterminalSymbol(DefaultSymbolType(3));
  rrGrammarRef.addNonterminalSymbol(DefaultSymbolType(4));
  rrGrammarRef.addTerminalSymbol(DefaultSymbolType("a"));
  rrGrammarRef.addTerminalSymbol(DefaultSymbolType("b"));

  rrGrammarRef.addRule(DefaultSymbolType(5), ext::make_pair(DefaultSymbolType("a"), DefaultSymbolType(2)));
  rrGrammarRef.addRule(DefaultSymbolType(2), ext::make_pair(DefaultSymbolType("b"), DefaultSymbolType(3)));
  rrGrammarRef.addRule(DefaultSymbolType(3), DefaultSymbolType("a"));
  rrGrammarRef.addRule(DefaultSymbolType(3), ext::make_pair(DefaultSymbolType("a"), DefaultSymbolType(4)));

  CPPUNIT_ASSERT(rrGrammarRef == rrGrammar);
}
