#include "LevenshteinDynamicProgrammingTest.h"

#include <string/LinearString.h>
#include <stringology/simulations/LevenshteinDynamicProgramming.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LevenshteinDynamicProgrammingTest, "dynamic programming" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LevenshteinDynamicProgrammingTest );

void LevenshteinDynamicProgrammingTest::testTableConstruction() {
  auto text = string::LinearString<>("adcabcaabadbbca");
  auto pattern = string::LinearString<>("adbbca");

  ext::vector<ext::vector<unsigned int>> expected_result = {
    ext::vector<unsigned int>({0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}),
    ext::vector<unsigned int>({1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0}),
    ext::vector<unsigned int>({2, 1, 0, 1, 1, 1, 2, 1, 1, 1, 1, 0, 1, 2, 2, 1}),
    ext::vector<unsigned int>({3, 2, 1, 1, 2, 1, 2, 2, 2, 1, 2, 1, 0, 1, 2, 2}),
    ext::vector<unsigned int>({4, 3, 2, 2, 2, 2, 2, 3, 3, 2, 2, 2, 1, 0, 1, 2}),
    ext::vector<unsigned int>({5, 4, 3, 2, 3, 3, 2, 3, 4, 3, 3, 3, 2, 1, 0, 1}),
    ext::vector<unsigned int>({6, 5, 4, 3, 2, 4, 3, 2, 3, 4, 3, 4, 3, 2, 1, 0}),
  };

  CPPUNIT_ASSERT(expected_result == stringology::simulations::LevenshteinDynamicProgramming::compute_table(text, pattern));
}

void LevenshteinDynamicProgrammingTest::testSearch() {
  auto text = string::LinearString<>("adcabcaabadbbca");
  auto pattern = string::LinearString<>("adbbca");

  ext::set<unsigned int> expected_result = {2, 3, 5, 6, 7, 9, 11, 12, 13, 14};
  auto result = stringology::simulations::LevenshteinDynamicProgramming::search(text, pattern, 3);
  CPPUNIT_ASSERT(expected_result == result);
}

void LevenshteinDynamicProgrammingTest::setUp() { }

void LevenshteinDynamicProgrammingTest::tearDown() { }
