/*
 * UniqueObject.cpp
 *
 *  Created on: Nov 11, 2016
 *      Author: Jan Travicek
 */

#include "UniqueObject.h"
#include <sstream>

#include <registration/ValuePrinterRegistration.hpp>

namespace object {

UniqueObject::UniqueObject(Object symbol, primitive::Integer id) : m_object(std::move(symbol)), m_id(std::move(id)) {
}

ObjectBase* UniqueObject::clone ( ) const & {
	return new UniqueObject(*this);
}

ObjectBase* UniqueObject::clone() && {
	return new UniqueObject(std::move(*this));
}

const Object & UniqueObject::getObject ( ) const & {
	return m_object;
}

Object && UniqueObject::getObject ( ) && {
	return std::move ( m_object );
}

const primitive::Integer& UniqueObject::getId() const {
	return m_id;
}

primitive::Integer& UniqueObject::getId() {
	return m_id;
}

int UniqueObject::compare(const UniqueObject& other) const {
	int res = m_object.getData().compare(other.m_object.getData());
	if(res == 0) res = m_id.compare(other.m_id);
	return res;
}

void UniqueObject::operator>>(std::ostream& out) const {
	out << "(UniqueObject " << m_object << ", " << m_id << ")";
}

UniqueObject::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << m_object;
	ss << ", ";
	ss << m_id;
	ss << "}";
	return std::move(ss).str();
}

ObjectBase* UniqueObject::inc() && {
	std::move(m_id).inc();
	return NULL;
}

} /* namespace object */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < object::UniqueObject > ( );

} /* namespace */
