/*
 * Void.h
 *
 * This file is part of Algorithms library toolkit.
 * Copyright (C) 2017 Jan Travnicek (jan.travnicek@fit.cvut.cz)

 * Algorithms library toolkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Algorithms library toolkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef VOID_H_
#define VOID_H_

#include "ObjectBase.h"

namespace object {

/**
 * \brief
 * Represents void object.
 */
class Void final : public ObjectBase {
public:
	/**
	 * \brief
	 * Creates a new instance of void object
	 */
	explicit Void ( );

	/**
	 * @copydoc object::ObjectBase::clone ( ) const &
	 */
	virtual ObjectBase * clone ( ) const & override;

	/**
	 * @copydoc object::ObjectBase::clone ( ) &&
	 */
	virtual ObjectBase * clone ( ) && override;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::compare ( const ObjectBase & ) const
	 */
	virtual int compare ( const ObjectBase & other ) const override {
		if ( ext::type_index ( typeid ( * this ) ) == ext::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return ext::type_index ( typeid ( * this ) ) - ext::type_index ( typeid ( other ) );
	}

	/**
	 * The actual compare method
	 *
	 * \param other the other instance
	 *
	 * \returns the actual relation between two by type same objects
	 */
	int compare ( const Void & other ) const;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator >> ( std::ostream & ) const
	 */
	virtual void operator >>( std::ostream & out ) const override;

	/**
	 * @copydoc base::CommonBase < ObjectBase >::operator std::string ( ) const
	 */
	virtual explicit operator std::string ( ) const override;

	/**
	 * \brief
	 * Singleton instance of the void object.
	 */
	static Void VOID;

	/**
	 * @copydoc object::ObjectBase::inc ( ) &&
	 */
	virtual ObjectBase * inc ( ) && override;
};

} /* namespace object */

#endif /* VOID_H_ */
