/*
 * pair.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __PAIR_HPP_
#define __PAIR_HPP_

#include <sstream>
#include <string>

#include "type_traits.hpp"
#include "utility.hpp"
#include "compare.hpp"

namespace ext {

template < class T, class R >
class pair : public std::pair < T, R > {
public:
	using std::pair < T, R >::pair;
	using std::pair < T, R >::operator =;
#ifndef __clang__
	pair ( const pair & other ) = default;

	pair ( pair && other ) = default;

	pair & operator = ( pair && other ) = default;

	pair & operator = ( const pair & other ) = default;
#endif
};

template<typename _T1, typename _T2>
constexpr ext::pair < typename std::__decay_and_strip < _T1 >::__type, typename std::__decay_and_strip < _T2 >::__type > make_pair ( _T1 && __x, _T2 && __y ) {
	typedef typename std::__decay_and_strip < _T1 >::__type __ds_type1;
	typedef typename std::__decay_and_strip < _T2 >::__type __ds_type2;
	typedef pair < __ds_type1, __ds_type2 > __pair_type;
	return __pair_type ( std::forward < _T1 > ( __x ), std::forward < _T2 > ( __y ) );
}

template< class T, class R >
std::ostream& operator<<(std::ostream& out, const ext::pair<T, R>& pair) {
	out << "(" << pair.first << ", " << pair.second << ")";
	return out;
}

template < class T, class R >
struct compare < ext::pair < T, R > > {
	int operator ( ) ( const ext::pair < T, R > & first, const ext::pair < T, R > & second ) const {
		static compare < typename std::decay < T >::type > compT;
		static compare < typename std::decay < R >::type > compR;

		int res = compT(first.first, second.first);
		if(res == 0) res = compR(first.second, second.second);
		return res;
	}
};

template < class T, class R >
std::string to_string ( const ext::pair < T, R > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __PAIR_HPP_ */
