#include <alib/list>
#include "TreeTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "tree/Tree.h"

#include "factory/StringDataFactory.hpp"

#include "alphabet/BlankSymbol.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( TreeTest, "tree" );
CPPUNIT_TEST_SUITE_REGISTRATION ( TreeTest );

void TreeTest::setUp ( ) {
}

void TreeTest::tearDown ( ) {
}

void TreeTest::testEqual ( ) {
	{
		std::string input = "RANKED_TREE a 0";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		std::cout << input << std::endl;
		std::cout << output << std::endl;
		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "UNRANKED_TREE a |";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "RANKED_TREE a 2 a 0 a 1 a 0";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "UNRANKED_TREE a a | a a | | |";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "RANKED_PATTERN a 2 #S a 1 a 0";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		std::cout << output << std::endl;
		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
	{
		std::string input = "UNRANKED_PATTERN a #S | a a | | |";
		tree::Tree tree = factory::StringDataFactory::fromString ( input );

		std::string output = factory::StringDataFactory::toString ( tree );

		std::cout << output << std::endl;
		CPPUNIT_ASSERT ( input == output );

		tree::Tree tree2 = factory::StringDataFactory::fromString ( output );

		CPPUNIT_ASSERT ( tree == tree2 );
	}
}
